# Generated from /Users/vijay/work/soda/code/soda-core/soda/core/soda/sodacl/antlr/SodaCLAntlr.g4 by ANTLR 4.11.1
# encoding: utf-8
from antlr4 import *
from io import StringIO
import sys
if sys.version_info[1] > 5:
	from typing import TextIO
else:
	from typing.io import TextIO

def serializedATN():
    return [
        4,1,57,391,2,0,7,0,2,1,7,1,2,2,7,2,2,3,7,3,2,4,7,4,2,5,7,5,2,6,7,
        6,2,7,7,7,2,8,7,8,2,9,7,9,2,10,7,10,2,11,7,11,2,12,7,12,2,13,7,13,
        2,14,7,14,2,15,7,15,2,16,7,16,2,17,7,17,2,18,7,18,2,19,7,19,2,20,
        7,20,2,21,7,21,2,22,7,22,2,23,7,23,2,24,7,24,2,25,7,25,2,26,7,26,
        2,27,7,27,2,28,7,28,2,29,7,29,2,30,7,30,2,31,7,31,2,32,7,32,2,33,
        7,33,2,34,7,34,2,35,7,35,2,36,7,36,2,37,7,37,2,38,7,38,2,39,7,39,
        2,40,7,40,2,41,7,41,1,0,1,0,1,0,1,0,1,0,1,0,3,0,91,8,0,1,1,1,1,1,
        1,1,1,3,1,97,8,1,1,1,1,1,1,1,1,1,3,1,103,8,1,1,1,1,1,1,2,1,2,1,2,
        1,2,1,2,1,3,1,3,1,3,1,4,1,4,1,4,1,5,1,5,1,5,1,6,1,6,1,6,1,6,1,6,
        3,6,126,8,6,1,6,1,6,1,6,1,6,3,6,132,8,6,1,6,1,6,1,7,1,7,3,7,138,
        8,7,1,7,1,7,1,7,1,7,3,7,144,8,7,3,7,146,8,7,1,7,1,7,1,8,1,8,1,8,
        1,8,1,9,1,9,1,9,1,9,1,9,3,9,159,8,9,1,9,3,9,162,8,9,1,9,1,9,1,9,
        1,10,1,10,1,10,1,10,1,10,1,10,1,10,3,10,174,8,10,1,11,1,11,1,12,
        1,12,1,13,1,13,1,13,1,14,1,14,1,15,1,15,3,15,187,8,15,1,16,1,16,
        1,17,1,17,1,17,1,17,1,17,5,17,196,8,17,10,17,12,17,199,9,17,1,17,
        1,17,1,18,1,18,3,18,205,8,18,1,19,1,19,3,19,209,8,19,1,20,1,20,3,
        20,213,8,20,1,20,1,20,1,20,3,20,218,8,20,1,20,1,20,1,20,1,20,1,20,
        1,20,3,20,226,8,20,1,21,1,21,1,21,1,21,1,22,1,22,1,22,1,22,1,22,
        1,22,1,22,1,22,1,22,4,22,241,8,22,11,22,12,22,242,1,22,1,22,1,23,
        1,23,1,24,1,24,1,25,1,25,1,26,1,26,3,26,255,8,26,1,26,3,26,258,8,
        26,1,26,1,26,3,26,262,8,26,1,27,1,27,1,27,4,27,267,8,27,11,27,12,
        27,268,1,27,3,27,272,8,27,1,28,1,28,1,28,1,28,1,28,1,28,1,28,1,28,
        1,28,1,28,1,28,1,28,1,28,1,28,1,28,1,28,1,28,5,28,291,8,28,10,28,
        12,28,294,9,28,1,28,1,28,1,28,1,28,1,28,1,28,1,28,1,28,1,28,1,28,
        1,28,5,28,307,8,28,10,28,12,28,310,9,28,1,28,1,28,3,28,314,8,28,
        1,29,1,29,1,30,1,30,1,31,1,31,1,31,1,31,1,31,3,31,325,8,31,1,32,
        1,32,1,32,1,32,1,32,3,32,332,8,32,1,32,1,32,1,33,1,33,1,33,1,33,
        1,34,1,34,1,34,1,34,1,34,1,34,1,34,1,35,1,35,1,35,1,35,1,35,1,36,
        1,36,1,36,1,36,1,36,1,36,1,36,1,36,1,36,1,36,3,36,362,8,36,1,37,
        1,37,1,37,1,37,1,37,1,38,3,38,370,8,38,1,38,1,38,1,39,1,39,1,39,
        1,39,3,39,378,8,39,1,39,3,39,381,8,39,1,39,1,39,3,39,385,8,39,1,
        40,1,40,1,41,1,41,1,41,0,0,42,0,2,4,6,8,10,12,14,16,18,20,22,24,
        26,28,30,32,34,36,38,40,42,44,46,48,50,52,54,56,58,60,62,64,66,68,
        70,72,74,76,78,80,82,0,9,1,0,32,34,2,0,35,35,39,39,2,0,36,36,40,
        40,1,0,27,29,2,0,47,47,50,50,1,0,45,51,1,0,10,12,1,0,43,44,2,0,32,
        34,52,54,392,0,90,1,0,0,0,2,92,1,0,0,0,4,106,1,0,0,0,6,111,1,0,0,
        0,8,114,1,0,0,0,10,117,1,0,0,0,12,120,1,0,0,0,14,137,1,0,0,0,16,
        149,1,0,0,0,18,153,1,0,0,0,20,173,1,0,0,0,22,175,1,0,0,0,24,177,
        1,0,0,0,26,179,1,0,0,0,28,182,1,0,0,0,30,184,1,0,0,0,32,188,1,0,
        0,0,34,190,1,0,0,0,36,204,1,0,0,0,38,208,1,0,0,0,40,212,1,0,0,0,
        42,227,1,0,0,0,44,240,1,0,0,0,46,246,1,0,0,0,48,248,1,0,0,0,50,250,
        1,0,0,0,52,261,1,0,0,0,54,266,1,0,0,0,56,313,1,0,0,0,58,315,1,0,
        0,0,60,317,1,0,0,0,62,324,1,0,0,0,64,326,1,0,0,0,66,335,1,0,0,0,
        68,339,1,0,0,0,70,346,1,0,0,0,72,361,1,0,0,0,74,363,1,0,0,0,76,369,
        1,0,0,0,78,384,1,0,0,0,80,386,1,0,0,0,82,388,1,0,0,0,84,91,3,8,4,
        0,85,91,3,12,6,0,86,91,3,14,7,0,87,91,3,56,28,0,88,91,3,2,1,0,89,
        91,3,10,5,0,90,84,1,0,0,0,90,85,1,0,0,0,90,86,1,0,0,0,90,87,1,0,
        0,0,90,88,1,0,0,0,90,89,1,0,0,0,91,1,1,0,0,0,92,93,5,1,0,0,93,94,
        5,57,0,0,94,96,3,82,41,0,95,97,3,4,2,0,96,95,1,0,0,0,96,97,1,0,0,
        0,97,102,1,0,0,0,98,99,5,57,0,0,99,100,5,50,0,0,100,101,5,57,0,0,
        101,103,3,54,27,0,102,98,1,0,0,0,102,103,1,0,0,0,103,104,1,0,0,0,
        104,105,5,0,0,1,105,3,1,0,0,0,106,107,5,57,0,0,107,108,5,2,0,0,108,
        109,5,57,0,0,109,110,3,82,41,0,110,5,1,0,0,0,111,112,5,57,0,0,112,
        113,5,27,0,0,113,7,1,0,0,0,114,115,5,3,0,0,115,116,5,0,0,1,116,9,
        1,0,0,0,117,118,5,4,0,0,118,119,5,0,0,1,119,11,1,0,0,0,120,121,5,
        5,0,0,121,122,5,57,0,0,122,125,3,82,41,0,123,124,5,57,0,0,124,126,
        3,66,33,0,125,123,1,0,0,0,125,126,1,0,0,0,126,131,1,0,0,0,127,128,
        5,57,0,0,128,129,5,26,0,0,129,130,5,57,0,0,130,132,3,82,41,0,131,
        127,1,0,0,0,131,132,1,0,0,0,132,133,1,0,0,0,133,134,5,0,0,1,134,
        13,1,0,0,0,135,138,3,18,9,0,136,138,3,28,14,0,137,135,1,0,0,0,137,
        136,1,0,0,0,137,138,1,0,0,0,138,139,1,0,0,0,139,145,3,30,15,0,140,
        143,5,57,0,0,141,144,3,38,19,0,142,144,3,16,8,0,143,141,1,0,0,0,
        143,142,1,0,0,0,144,146,1,0,0,0,145,140,1,0,0,0,145,146,1,0,0,0,
        146,147,1,0,0,0,147,148,5,0,0,1,148,15,1,0,0,0,149,150,5,50,0,0,
        150,151,5,57,0,0,151,152,5,6,0,0,152,17,1,0,0,0,153,154,5,30,0,0,
        154,158,5,57,0,0,155,156,3,20,10,0,156,157,5,57,0,0,157,159,1,0,
        0,0,158,155,1,0,0,0,158,159,1,0,0,0,159,161,1,0,0,0,160,162,3,26,
        13,0,161,160,1,0,0,0,161,162,1,0,0,0,162,163,1,0,0,0,163,164,5,22,
        0,0,164,165,5,57,0,0,165,19,1,0,0,0,166,167,3,22,11,0,167,168,5,
        57,0,0,168,169,5,31,0,0,169,170,5,57,0,0,170,171,3,80,40,0,171,174,
        1,0,0,0,172,174,3,24,12,0,173,166,1,0,0,0,173,172,1,0,0,0,174,21,
        1,0,0,0,175,176,7,0,0,0,176,23,1,0,0,0,177,178,5,7,0,0,178,25,1,
        0,0,0,179,180,5,8,0,0,180,181,5,57,0,0,181,27,1,0,0,0,182,183,5,
        9,0,0,183,29,1,0,0,0,184,186,3,32,16,0,185,187,3,34,17,0,186,185,
        1,0,0,0,186,187,1,0,0,0,187,31,1,0,0,0,188,189,3,82,41,0,189,33,
        1,0,0,0,190,191,5,39,0,0,191,197,3,36,18,0,192,193,5,41,0,0,193,
        194,5,57,0,0,194,196,3,36,18,0,195,192,1,0,0,0,196,199,1,0,0,0,197,
        195,1,0,0,0,197,198,1,0,0,0,198,200,1,0,0,0,199,197,1,0,0,0,200,
        201,5,40,0,0,201,35,1,0,0,0,202,205,3,76,38,0,203,205,3,82,41,0,
        204,202,1,0,0,0,204,203,1,0,0,0,205,37,1,0,0,0,206,209,3,42,21,0,
        207,209,3,40,20,0,208,206,1,0,0,0,208,207,1,0,0,0,209,39,1,0,0,0,
        210,211,5,25,0,0,211,213,5,57,0,0,212,210,1,0,0,0,212,213,1,0,0,
        0,213,214,1,0,0,0,214,215,5,24,0,0,215,217,5,57,0,0,216,218,7,1,
        0,0,217,216,1,0,0,0,217,218,1,0,0,0,218,219,1,0,0,0,219,220,3,52,
        26,0,220,221,5,57,0,0,221,222,5,23,0,0,222,223,5,57,0,0,223,225,
        3,52,26,0,224,226,7,2,0,0,225,224,1,0,0,0,225,226,1,0,0,0,226,41,
        1,0,0,0,227,228,3,50,25,0,228,229,5,57,0,0,229,230,3,52,26,0,230,
        43,1,0,0,0,231,232,3,46,23,0,232,233,5,57,0,0,233,234,3,48,24,0,
        234,235,5,57,0,0,235,236,3,52,26,0,236,237,5,57,0,0,237,238,3,48,
        24,0,238,239,5,57,0,0,239,241,1,0,0,0,240,231,1,0,0,0,241,242,1,
        0,0,0,242,240,1,0,0,0,242,243,1,0,0,0,243,244,1,0,0,0,244,245,3,
        46,23,0,245,45,1,0,0,0,246,247,7,3,0,0,247,47,1,0,0,0,248,249,7,
        4,0,0,249,49,1,0,0,0,250,251,7,5,0,0,251,51,1,0,0,0,252,257,3,76,
        38,0,253,255,5,57,0,0,254,253,1,0,0,0,254,255,1,0,0,0,255,256,1,
        0,0,0,256,258,5,42,0,0,257,254,1,0,0,0,257,258,1,0,0,0,258,262,1,
        0,0,0,259,262,3,54,27,0,260,262,5,54,0,0,261,252,1,0,0,0,261,259,
        1,0,0,0,261,260,1,0,0,0,262,53,1,0,0,0,263,264,3,80,40,0,264,265,
        7,6,0,0,265,267,1,0,0,0,266,263,1,0,0,0,267,268,1,0,0,0,268,266,
        1,0,0,0,268,269,1,0,0,0,269,271,1,0,0,0,270,272,3,80,40,0,271,270,
        1,0,0,0,271,272,1,0,0,0,272,55,1,0,0,0,273,274,5,13,0,0,274,275,
        5,57,0,0,275,276,3,58,29,0,276,277,5,57,0,0,277,278,5,14,0,0,278,
        279,5,57,0,0,279,280,3,82,41,0,280,281,5,57,0,0,281,282,3,60,30,
        0,282,314,1,0,0,0,283,284,5,13,0,0,284,285,5,57,0,0,285,286,5,39,
        0,0,286,292,3,58,29,0,287,288,5,41,0,0,288,289,5,57,0,0,289,291,
        3,58,29,0,290,287,1,0,0,0,291,294,1,0,0,0,292,290,1,0,0,0,292,293,
        1,0,0,0,293,295,1,0,0,0,294,292,1,0,0,0,295,296,5,40,0,0,296,297,
        5,57,0,0,297,298,5,14,0,0,298,299,5,57,0,0,299,300,3,82,41,0,300,
        301,5,57,0,0,301,302,5,39,0,0,302,308,3,60,30,0,303,304,5,41,0,0,
        304,305,5,57,0,0,305,307,3,60,30,0,306,303,1,0,0,0,307,310,1,0,0,
        0,308,306,1,0,0,0,308,309,1,0,0,0,309,311,1,0,0,0,310,308,1,0,0,
        0,311,312,5,40,0,0,312,314,1,0,0,0,313,273,1,0,0,0,313,283,1,0,0,
        0,314,57,1,0,0,0,315,316,3,82,41,0,316,59,1,0,0,0,317,318,3,82,41,
        0,318,61,1,0,0,0,319,325,3,64,32,0,320,325,3,70,35,0,321,325,3,68,
        34,0,322,325,3,72,36,0,323,325,3,74,37,0,324,319,1,0,0,0,324,320,
        1,0,0,0,324,321,1,0,0,0,324,322,1,0,0,0,324,323,1,0,0,0,325,63,1,
        0,0,0,326,327,5,15,0,0,327,328,5,57,0,0,328,331,3,82,41,0,329,330,
        5,57,0,0,330,332,3,66,33,0,331,329,1,0,0,0,331,332,1,0,0,0,332,333,
        1,0,0,0,333,334,5,0,0,1,334,65,1,0,0,0,335,336,5,35,0,0,336,337,
        3,82,41,0,337,338,5,36,0,0,338,67,1,0,0,0,339,340,5,16,0,0,340,341,
        5,57,0,0,341,342,3,82,41,0,342,343,5,57,0,0,343,344,3,66,33,0,344,
        345,5,0,0,1,345,69,1,0,0,0,346,347,5,17,0,0,347,348,5,57,0,0,348,
        349,3,82,41,0,349,350,5,0,0,1,350,71,1,0,0,0,351,352,5,18,0,0,352,
        353,5,57,0,0,353,354,3,82,41,0,354,355,5,0,0,1,355,362,1,0,0,0,356,
        357,5,19,0,0,357,358,5,57,0,0,358,359,3,82,41,0,359,360,5,0,0,1,
        360,362,1,0,0,0,361,351,1,0,0,0,361,356,1,0,0,0,362,73,1,0,0,0,363,
        364,5,20,0,0,364,365,5,57,0,0,365,366,3,82,41,0,366,367,5,0,0,1,
        367,75,1,0,0,0,368,370,7,7,0,0,369,368,1,0,0,0,369,370,1,0,0,0,370,
        371,1,0,0,0,371,372,3,78,39,0,372,77,1,0,0,0,373,385,3,80,40,0,374,
        375,5,56,0,0,375,377,5,21,0,0,376,378,5,56,0,0,377,376,1,0,0,0,377,
        378,1,0,0,0,378,385,1,0,0,0,379,381,5,56,0,0,380,379,1,0,0,0,380,
        381,1,0,0,0,381,382,1,0,0,0,382,383,5,21,0,0,383,385,5,56,0,0,384,
        373,1,0,0,0,384,374,1,0,0,0,384,380,1,0,0,0,385,79,1,0,0,0,386,387,
        5,56,0,0,387,81,1,0,0,0,388,389,7,8,0,0,389,83,1,0,0,0,34,90,96,
        102,125,131,137,143,145,158,161,173,186,197,204,208,212,217,225,
        242,254,257,261,268,271,292,308,313,324,331,361,369,377,380,384
    ]

class SodaCLAntlrParser ( Parser ):

    grammarFileName = "SodaCLAntlr.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "'freshness using'", "'with'", "'failed rows'", 
                     "'group by'", "'row_count same as'", "'default'", "'same day last week'", 
                     "'percent'", "'anomaly score for '", "'d'", "'h'", 
                     "'m'", "'values in'", "'must exist in'", "'checks for'", 
                     "'filter'", "'configurations for'", "'for each dataset'", 
                     "'for each table'", "'for each column'", "'.'", "'for'", 
                     "'and'", "'between'", "'not'", "'in'", "'warn'", "'fail'", 
                     "'pass'", "'change'", "'last'", "'avg'", "'min'", "'max'", 
                     "'['", "']'", "'{'", "'}'", "'('", "')'", "','", "'%'", 
                     "'+'", "'-'", "'!='", "'<>'", "'<='", "'>='", "'='", 
                     "'<'", "'>'", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "' '" ]

    symbolicNames = [ "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "FOR", "AND", "BETWEEN", 
                      "NOT", "IN", "WARN", "FAIL", "PASS", "CHANGE", "LAST", 
                      "AVG", "MIN", "MAX", "SQUARE_LEFT", "SQUARE_RIGHT", 
                      "CURLY_LEFT", "CURLY_RIGHT", "ROUND_LEFT", "ROUND_RIGHT", 
                      "COMMA", "PERCENT", "PLUS", "MINUS", "NOT_EQUAL", 
                      "NOT_EQUAL_SQL", "LTE", "GTE", "EQUAL", "LT", "GT", 
                      "IDENTIFIER_DOUBLE_QUOTE", "IDENTIFIER_BACKTICK", 
                      "IDENTIFIER_UNQUOTED", "STRING", "DIGITS", "S" ]

    RULE_check = 0
    RULE_freshness_check = 1
    RULE_freshness_variable = 2
    RULE_warn_qualifier = 3
    RULE_failed_rows_check = 4
    RULE_group_by_check = 5
    RULE_row_count_comparison_check = 6
    RULE_metric_check = 7
    RULE_default_anomaly_threshold = 8
    RULE_change_over_time = 9
    RULE_change_over_time_config = 10
    RULE_change_aggregation = 11
    RULE_same_day_last_week = 12
    RULE_percent = 13
    RULE_anomaly_score = 14
    RULE_metric = 15
    RULE_metric_name = 16
    RULE_metric_args = 17
    RULE_metric_arg = 18
    RULE_threshold = 19
    RULE_between_threshold = 20
    RULE_comparator_threshold = 21
    RULE_zones_threshold = 22
    RULE_outcome = 23
    RULE_zone_comparator = 24
    RULE_comparator = 25
    RULE_threshold_value = 26
    RULE_freshness_threshold_value = 27
    RULE_reference_check = 28
    RULE_source_column_name = 29
    RULE_target_column_name = 30
    RULE_section_header = 31
    RULE_table_checks_header = 32
    RULE_partition_name = 33
    RULE_table_filter_header = 34
    RULE_column_configurations_header = 35
    RULE_checks_for_each_dataset_header = 36
    RULE_checks_for_each_column_header = 37
    RULE_signed_number = 38
    RULE_number = 39
    RULE_integer = 40
    RULE_identifier = 41

    ruleNames =  [ "check", "freshness_check", "freshness_variable", "warn_qualifier", 
                   "failed_rows_check", "group_by_check", "row_count_comparison_check", 
                   "metric_check", "default_anomaly_threshold", "change_over_time", 
                   "change_over_time_config", "change_aggregation", "same_day_last_week", 
                   "percent", "anomaly_score", "metric", "metric_name", 
                   "metric_args", "metric_arg", "threshold", "between_threshold", 
                   "comparator_threshold", "zones_threshold", "outcome", 
                   "zone_comparator", "comparator", "threshold_value", "freshness_threshold_value", 
                   "reference_check", "source_column_name", "target_column_name", 
                   "section_header", "table_checks_header", "partition_name", 
                   "table_filter_header", "column_configurations_header", 
                   "checks_for_each_dataset_header", "checks_for_each_column_header", 
                   "signed_number", "number", "integer", "identifier" ]

    EOF = Token.EOF
    T__0=1
    T__1=2
    T__2=3
    T__3=4
    T__4=5
    T__5=6
    T__6=7
    T__7=8
    T__8=9
    T__9=10
    T__10=11
    T__11=12
    T__12=13
    T__13=14
    T__14=15
    T__15=16
    T__16=17
    T__17=18
    T__18=19
    T__19=20
    T__20=21
    FOR=22
    AND=23
    BETWEEN=24
    NOT=25
    IN=26
    WARN=27
    FAIL=28
    PASS=29
    CHANGE=30
    LAST=31
    AVG=32
    MIN=33
    MAX=34
    SQUARE_LEFT=35
    SQUARE_RIGHT=36
    CURLY_LEFT=37
    CURLY_RIGHT=38
    ROUND_LEFT=39
    ROUND_RIGHT=40
    COMMA=41
    PERCENT=42
    PLUS=43
    MINUS=44
    NOT_EQUAL=45
    NOT_EQUAL_SQL=46
    LTE=47
    GTE=48
    EQUAL=49
    LT=50
    GT=51
    IDENTIFIER_DOUBLE_QUOTE=52
    IDENTIFIER_BACKTICK=53
    IDENTIFIER_UNQUOTED=54
    STRING=55
    DIGITS=56
    S=57

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.11.1")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None




    class CheckContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def failed_rows_check(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Failed_rows_checkContext,0)


        def row_count_comparison_check(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Row_count_comparison_checkContext,0)


        def metric_check(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Metric_checkContext,0)


        def reference_check(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Reference_checkContext,0)


        def freshness_check(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Freshness_checkContext,0)


        def group_by_check(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Group_by_checkContext,0)


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_check

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCheck" ):
                listener.enterCheck(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCheck" ):
                listener.exitCheck(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitCheck" ):
                return visitor.visitCheck(self)
            else:
                return visitor.visitChildren(self)




    def check(self):

        localctx = SodaCLAntlrParser.CheckContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_check)
        try:
            self.state = 90
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [3]:
                self.enterOuterAlt(localctx, 1)
                self.state = 84
                self.failed_rows_check()
                pass
            elif token in [5]:
                self.enterOuterAlt(localctx, 2)
                self.state = 85
                self.row_count_comparison_check()
                pass
            elif token in [9, 30, 32, 33, 34, 52, 53, 54]:
                self.enterOuterAlt(localctx, 3)
                self.state = 86
                self.metric_check()
                pass
            elif token in [13]:
                self.enterOuterAlt(localctx, 4)
                self.state = 87
                self.reference_check()
                pass
            elif token in [1]:
                self.enterOuterAlt(localctx, 5)
                self.state = 88
                self.freshness_check()
                pass
            elif token in [4]:
                self.enterOuterAlt(localctx, 6)
                self.state = 89
                self.group_by_check()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Freshness_checkContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def S(self, i:int=None):
            if i is None:
                return self.getTokens(SodaCLAntlrParser.S)
            else:
                return self.getToken(SodaCLAntlrParser.S, i)

        def identifier(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.IdentifierContext,0)


        def EOF(self):
            return self.getToken(SodaCLAntlrParser.EOF, 0)

        def freshness_variable(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Freshness_variableContext,0)


        def LT(self):
            return self.getToken(SodaCLAntlrParser.LT, 0)

        def freshness_threshold_value(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Freshness_threshold_valueContext,0)


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_freshness_check

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFreshness_check" ):
                listener.enterFreshness_check(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFreshness_check" ):
                listener.exitFreshness_check(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFreshness_check" ):
                return visitor.visitFreshness_check(self)
            else:
                return visitor.visitChildren(self)




    def freshness_check(self):

        localctx = SodaCLAntlrParser.Freshness_checkContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_freshness_check)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 92
            self.match(SodaCLAntlrParser.T__0)
            self.state = 93
            self.match(SodaCLAntlrParser.S)
            self.state = 94
            self.identifier()
            self.state = 96
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,1,self._ctx)
            if la_ == 1:
                self.state = 95
                self.freshness_variable()


            self.state = 102
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==57:
                self.state = 98
                self.match(SodaCLAntlrParser.S)
                self.state = 99
                self.match(SodaCLAntlrParser.LT)
                self.state = 100
                self.match(SodaCLAntlrParser.S)
                self.state = 101
                self.freshness_threshold_value()


            self.state = 104
            self.match(SodaCLAntlrParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Freshness_variableContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def S(self, i:int=None):
            if i is None:
                return self.getTokens(SodaCLAntlrParser.S)
            else:
                return self.getToken(SodaCLAntlrParser.S, i)

        def identifier(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.IdentifierContext,0)


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_freshness_variable

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFreshness_variable" ):
                listener.enterFreshness_variable(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFreshness_variable" ):
                listener.exitFreshness_variable(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFreshness_variable" ):
                return visitor.visitFreshness_variable(self)
            else:
                return visitor.visitChildren(self)




    def freshness_variable(self):

        localctx = SodaCLAntlrParser.Freshness_variableContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_freshness_variable)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 106
            self.match(SodaCLAntlrParser.S)
            self.state = 107
            self.match(SodaCLAntlrParser.T__1)
            self.state = 108
            self.match(SodaCLAntlrParser.S)
            self.state = 109
            self.identifier()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Warn_qualifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def S(self):
            return self.getToken(SodaCLAntlrParser.S, 0)

        def WARN(self):
            return self.getToken(SodaCLAntlrParser.WARN, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_warn_qualifier

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterWarn_qualifier" ):
                listener.enterWarn_qualifier(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitWarn_qualifier" ):
                listener.exitWarn_qualifier(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitWarn_qualifier" ):
                return visitor.visitWarn_qualifier(self)
            else:
                return visitor.visitChildren(self)




    def warn_qualifier(self):

        localctx = SodaCLAntlrParser.Warn_qualifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_warn_qualifier)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 111
            self.match(SodaCLAntlrParser.S)
            self.state = 112
            self.match(SodaCLAntlrParser.WARN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Failed_rows_checkContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def EOF(self):
            return self.getToken(SodaCLAntlrParser.EOF, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_failed_rows_check

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFailed_rows_check" ):
                listener.enterFailed_rows_check(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFailed_rows_check" ):
                listener.exitFailed_rows_check(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFailed_rows_check" ):
                return visitor.visitFailed_rows_check(self)
            else:
                return visitor.visitChildren(self)




    def failed_rows_check(self):

        localctx = SodaCLAntlrParser.Failed_rows_checkContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_failed_rows_check)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 114
            self.match(SodaCLAntlrParser.T__2)
            self.state = 115
            self.match(SodaCLAntlrParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Group_by_checkContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def EOF(self):
            return self.getToken(SodaCLAntlrParser.EOF, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_group_by_check

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterGroup_by_check" ):
                listener.enterGroup_by_check(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitGroup_by_check" ):
                listener.exitGroup_by_check(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitGroup_by_check" ):
                return visitor.visitGroup_by_check(self)
            else:
                return visitor.visitChildren(self)




    def group_by_check(self):

        localctx = SodaCLAntlrParser.Group_by_checkContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_group_by_check)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 117
            self.match(SodaCLAntlrParser.T__3)
            self.state = 118
            self.match(SodaCLAntlrParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Row_count_comparison_checkContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def S(self, i:int=None):
            if i is None:
                return self.getTokens(SodaCLAntlrParser.S)
            else:
                return self.getToken(SodaCLAntlrParser.S, i)

        def identifier(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SodaCLAntlrParser.IdentifierContext)
            else:
                return self.getTypedRuleContext(SodaCLAntlrParser.IdentifierContext,i)


        def EOF(self):
            return self.getToken(SodaCLAntlrParser.EOF, 0)

        def partition_name(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Partition_nameContext,0)


        def IN(self):
            return self.getToken(SodaCLAntlrParser.IN, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_row_count_comparison_check

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterRow_count_comparison_check" ):
                listener.enterRow_count_comparison_check(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitRow_count_comparison_check" ):
                listener.exitRow_count_comparison_check(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitRow_count_comparison_check" ):
                return visitor.visitRow_count_comparison_check(self)
            else:
                return visitor.visitChildren(self)




    def row_count_comparison_check(self):

        localctx = SodaCLAntlrParser.Row_count_comparison_checkContext(self, self._ctx, self.state)
        self.enterRule(localctx, 12, self.RULE_row_count_comparison_check)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 120
            self.match(SodaCLAntlrParser.T__4)
            self.state = 121
            self.match(SodaCLAntlrParser.S)
            self.state = 122
            self.identifier()
            self.state = 125
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,3,self._ctx)
            if la_ == 1:
                self.state = 123
                self.match(SodaCLAntlrParser.S)
                self.state = 124
                self.partition_name()


            self.state = 131
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==57:
                self.state = 127
                self.match(SodaCLAntlrParser.S)
                self.state = 128
                self.match(SodaCLAntlrParser.IN)
                self.state = 129
                self.match(SodaCLAntlrParser.S)
                self.state = 130
                self.identifier()


            self.state = 133
            self.match(SodaCLAntlrParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Metric_checkContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def metric(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.MetricContext,0)


        def EOF(self):
            return self.getToken(SodaCLAntlrParser.EOF, 0)

        def change_over_time(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Change_over_timeContext,0)


        def anomaly_score(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Anomaly_scoreContext,0)


        def S(self):
            return self.getToken(SodaCLAntlrParser.S, 0)

        def threshold(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.ThresholdContext,0)


        def default_anomaly_threshold(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Default_anomaly_thresholdContext,0)


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_metric_check

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterMetric_check" ):
                listener.enterMetric_check(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitMetric_check" ):
                listener.exitMetric_check(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitMetric_check" ):
                return visitor.visitMetric_check(self)
            else:
                return visitor.visitChildren(self)




    def metric_check(self):

        localctx = SodaCLAntlrParser.Metric_checkContext(self, self._ctx, self.state)
        self.enterRule(localctx, 14, self.RULE_metric_check)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 137
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [30]:
                self.state = 135
                self.change_over_time()
                pass
            elif token in [9]:
                self.state = 136
                self.anomaly_score()
                pass
            elif token in [32, 33, 34, 52, 53, 54]:
                pass
            else:
                pass
            self.state = 139
            self.metric()
            self.state = 145
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==57:
                self.state = 140
                self.match(SodaCLAntlrParser.S)
                self.state = 143
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,6,self._ctx)
                if la_ == 1:
                    self.state = 141
                    self.threshold()
                    pass

                elif la_ == 2:
                    self.state = 142
                    self.default_anomaly_threshold()
                    pass




            self.state = 147
            self.match(SodaCLAntlrParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Default_anomaly_thresholdContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LT(self):
            return self.getToken(SodaCLAntlrParser.LT, 0)

        def S(self):
            return self.getToken(SodaCLAntlrParser.S, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_default_anomaly_threshold

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDefault_anomaly_threshold" ):
                listener.enterDefault_anomaly_threshold(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDefault_anomaly_threshold" ):
                listener.exitDefault_anomaly_threshold(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitDefault_anomaly_threshold" ):
                return visitor.visitDefault_anomaly_threshold(self)
            else:
                return visitor.visitChildren(self)




    def default_anomaly_threshold(self):

        localctx = SodaCLAntlrParser.Default_anomaly_thresholdContext(self, self._ctx, self.state)
        self.enterRule(localctx, 16, self.RULE_default_anomaly_threshold)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 149
            self.match(SodaCLAntlrParser.LT)
            self.state = 150
            self.match(SodaCLAntlrParser.S)
            self.state = 151
            self.match(SodaCLAntlrParser.T__5)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Change_over_timeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def CHANGE(self):
            return self.getToken(SodaCLAntlrParser.CHANGE, 0)

        def S(self, i:int=None):
            if i is None:
                return self.getTokens(SodaCLAntlrParser.S)
            else:
                return self.getToken(SodaCLAntlrParser.S, i)

        def FOR(self):
            return self.getToken(SodaCLAntlrParser.FOR, 0)

        def change_over_time_config(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Change_over_time_configContext,0)


        def percent(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.PercentContext,0)


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_change_over_time

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterChange_over_time" ):
                listener.enterChange_over_time(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitChange_over_time" ):
                listener.exitChange_over_time(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitChange_over_time" ):
                return visitor.visitChange_over_time(self)
            else:
                return visitor.visitChildren(self)




    def change_over_time(self):

        localctx = SodaCLAntlrParser.Change_over_timeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 18, self.RULE_change_over_time)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 153
            self.match(SodaCLAntlrParser.CHANGE)
            self.state = 154
            self.match(SodaCLAntlrParser.S)
            self.state = 158
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if ((_la) & ~0x3f) == 0 and ((1 << _la) & 30064771200) != 0:
                self.state = 155
                self.change_over_time_config()
                self.state = 156
                self.match(SodaCLAntlrParser.S)


            self.state = 161
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==8:
                self.state = 160
                self.percent()


            self.state = 163
            self.match(SodaCLAntlrParser.FOR)
            self.state = 164
            self.match(SodaCLAntlrParser.S)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Change_over_time_configContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def change_aggregation(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Change_aggregationContext,0)


        def S(self, i:int=None):
            if i is None:
                return self.getTokens(SodaCLAntlrParser.S)
            else:
                return self.getToken(SodaCLAntlrParser.S, i)

        def LAST(self):
            return self.getToken(SodaCLAntlrParser.LAST, 0)

        def integer(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.IntegerContext,0)


        def same_day_last_week(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Same_day_last_weekContext,0)


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_change_over_time_config

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterChange_over_time_config" ):
                listener.enterChange_over_time_config(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitChange_over_time_config" ):
                listener.exitChange_over_time_config(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitChange_over_time_config" ):
                return visitor.visitChange_over_time_config(self)
            else:
                return visitor.visitChildren(self)




    def change_over_time_config(self):

        localctx = SodaCLAntlrParser.Change_over_time_configContext(self, self._ctx, self.state)
        self.enterRule(localctx, 20, self.RULE_change_over_time_config)
        try:
            self.state = 173
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [32, 33, 34]:
                self.enterOuterAlt(localctx, 1)
                self.state = 166
                self.change_aggregation()
                self.state = 167
                self.match(SodaCLAntlrParser.S)
                self.state = 168
                self.match(SodaCLAntlrParser.LAST)
                self.state = 169
                self.match(SodaCLAntlrParser.S)
                self.state = 170
                self.integer()
                pass
            elif token in [7]:
                self.enterOuterAlt(localctx, 2)
                self.state = 172
                self.same_day_last_week()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Change_aggregationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def AVG(self):
            return self.getToken(SodaCLAntlrParser.AVG, 0)

        def MIN(self):
            return self.getToken(SodaCLAntlrParser.MIN, 0)

        def MAX(self):
            return self.getToken(SodaCLAntlrParser.MAX, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_change_aggregation

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterChange_aggregation" ):
                listener.enterChange_aggregation(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitChange_aggregation" ):
                listener.exitChange_aggregation(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitChange_aggregation" ):
                return visitor.visitChange_aggregation(self)
            else:
                return visitor.visitChildren(self)




    def change_aggregation(self):

        localctx = SodaCLAntlrParser.Change_aggregationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 22, self.RULE_change_aggregation)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 175
            _la = self._input.LA(1)
            if not(((_la) & ~0x3f) == 0 and ((1 << _la) & 30064771072) != 0):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Same_day_last_weekContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_same_day_last_week

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSame_day_last_week" ):
                listener.enterSame_day_last_week(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSame_day_last_week" ):
                listener.exitSame_day_last_week(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSame_day_last_week" ):
                return visitor.visitSame_day_last_week(self)
            else:
                return visitor.visitChildren(self)




    def same_day_last_week(self):

        localctx = SodaCLAntlrParser.Same_day_last_weekContext(self, self._ctx, self.state)
        self.enterRule(localctx, 24, self.RULE_same_day_last_week)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 177
            self.match(SodaCLAntlrParser.T__6)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PercentContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def S(self):
            return self.getToken(SodaCLAntlrParser.S, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_percent

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPercent" ):
                listener.enterPercent(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPercent" ):
                listener.exitPercent(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitPercent" ):
                return visitor.visitPercent(self)
            else:
                return visitor.visitChildren(self)




    def percent(self):

        localctx = SodaCLAntlrParser.PercentContext(self, self._ctx, self.state)
        self.enterRule(localctx, 26, self.RULE_percent)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 179
            self.match(SodaCLAntlrParser.T__7)
            self.state = 180
            self.match(SodaCLAntlrParser.S)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Anomaly_scoreContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_anomaly_score

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAnomaly_score" ):
                listener.enterAnomaly_score(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAnomaly_score" ):
                listener.exitAnomaly_score(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAnomaly_score" ):
                return visitor.visitAnomaly_score(self)
            else:
                return visitor.visitChildren(self)




    def anomaly_score(self):

        localctx = SodaCLAntlrParser.Anomaly_scoreContext(self, self._ctx, self.state)
        self.enterRule(localctx, 28, self.RULE_anomaly_score)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 182
            self.match(SodaCLAntlrParser.T__8)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class MetricContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def metric_name(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Metric_nameContext,0)


        def metric_args(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Metric_argsContext,0)


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_metric

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterMetric" ):
                listener.enterMetric(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitMetric" ):
                listener.exitMetric(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitMetric" ):
                return visitor.visitMetric(self)
            else:
                return visitor.visitChildren(self)




    def metric(self):

        localctx = SodaCLAntlrParser.MetricContext(self, self._ctx, self.state)
        self.enterRule(localctx, 30, self.RULE_metric)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 184
            self.metric_name()
            self.state = 186
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==39:
                self.state = 185
                self.metric_args()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Metric_nameContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def identifier(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.IdentifierContext,0)


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_metric_name

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterMetric_name" ):
                listener.enterMetric_name(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitMetric_name" ):
                listener.exitMetric_name(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitMetric_name" ):
                return visitor.visitMetric_name(self)
            else:
                return visitor.visitChildren(self)




    def metric_name(self):

        localctx = SodaCLAntlrParser.Metric_nameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 32, self.RULE_metric_name)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 188
            self.identifier()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Metric_argsContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ROUND_LEFT(self):
            return self.getToken(SodaCLAntlrParser.ROUND_LEFT, 0)

        def metric_arg(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SodaCLAntlrParser.Metric_argContext)
            else:
                return self.getTypedRuleContext(SodaCLAntlrParser.Metric_argContext,i)


        def ROUND_RIGHT(self):
            return self.getToken(SodaCLAntlrParser.ROUND_RIGHT, 0)

        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(SodaCLAntlrParser.COMMA)
            else:
                return self.getToken(SodaCLAntlrParser.COMMA, i)

        def S(self, i:int=None):
            if i is None:
                return self.getTokens(SodaCLAntlrParser.S)
            else:
                return self.getToken(SodaCLAntlrParser.S, i)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_metric_args

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterMetric_args" ):
                listener.enterMetric_args(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitMetric_args" ):
                listener.exitMetric_args(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitMetric_args" ):
                return visitor.visitMetric_args(self)
            else:
                return visitor.visitChildren(self)




    def metric_args(self):

        localctx = SodaCLAntlrParser.Metric_argsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 34, self.RULE_metric_args)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 190
            self.match(SodaCLAntlrParser.ROUND_LEFT)
            self.state = 191
            self.metric_arg()
            self.state = 197
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==41:
                self.state = 192
                self.match(SodaCLAntlrParser.COMMA)
                self.state = 193
                self.match(SodaCLAntlrParser.S)
                self.state = 194
                self.metric_arg()
                self.state = 199
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 200
            self.match(SodaCLAntlrParser.ROUND_RIGHT)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Metric_argContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def signed_number(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Signed_numberContext,0)


        def identifier(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.IdentifierContext,0)


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_metric_arg

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterMetric_arg" ):
                listener.enterMetric_arg(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitMetric_arg" ):
                listener.exitMetric_arg(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitMetric_arg" ):
                return visitor.visitMetric_arg(self)
            else:
                return visitor.visitChildren(self)




    def metric_arg(self):

        localctx = SodaCLAntlrParser.Metric_argContext(self, self._ctx, self.state)
        self.enterRule(localctx, 36, self.RULE_metric_arg)
        try:
            self.state = 204
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [21, 43, 44, 56]:
                self.enterOuterAlt(localctx, 1)
                self.state = 202
                self.signed_number()
                pass
            elif token in [32, 33, 34, 52, 53, 54]:
                self.enterOuterAlt(localctx, 2)
                self.state = 203
                self.identifier()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ThresholdContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def comparator_threshold(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Comparator_thresholdContext,0)


        def between_threshold(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Between_thresholdContext,0)


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_threshold

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterThreshold" ):
                listener.enterThreshold(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitThreshold" ):
                listener.exitThreshold(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitThreshold" ):
                return visitor.visitThreshold(self)
            else:
                return visitor.visitChildren(self)




    def threshold(self):

        localctx = SodaCLAntlrParser.ThresholdContext(self, self._ctx, self.state)
        self.enterRule(localctx, 38, self.RULE_threshold)
        try:
            self.state = 208
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [45, 46, 47, 48, 49, 50, 51]:
                self.enterOuterAlt(localctx, 1)
                self.state = 206
                self.comparator_threshold()
                pass
            elif token in [24, 25]:
                self.enterOuterAlt(localctx, 2)
                self.state = 207
                self.between_threshold()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Between_thresholdContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def BETWEEN(self):
            return self.getToken(SodaCLAntlrParser.BETWEEN, 0)

        def S(self, i:int=None):
            if i is None:
                return self.getTokens(SodaCLAntlrParser.S)
            else:
                return self.getToken(SodaCLAntlrParser.S, i)

        def threshold_value(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SodaCLAntlrParser.Threshold_valueContext)
            else:
                return self.getTypedRuleContext(SodaCLAntlrParser.Threshold_valueContext,i)


        def AND(self):
            return self.getToken(SodaCLAntlrParser.AND, 0)

        def NOT(self):
            return self.getToken(SodaCLAntlrParser.NOT, 0)

        def SQUARE_LEFT(self):
            return self.getToken(SodaCLAntlrParser.SQUARE_LEFT, 0)

        def ROUND_LEFT(self):
            return self.getToken(SodaCLAntlrParser.ROUND_LEFT, 0)

        def SQUARE_RIGHT(self):
            return self.getToken(SodaCLAntlrParser.SQUARE_RIGHT, 0)

        def ROUND_RIGHT(self):
            return self.getToken(SodaCLAntlrParser.ROUND_RIGHT, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_between_threshold

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBetween_threshold" ):
                listener.enterBetween_threshold(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBetween_threshold" ):
                listener.exitBetween_threshold(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitBetween_threshold" ):
                return visitor.visitBetween_threshold(self)
            else:
                return visitor.visitChildren(self)




    def between_threshold(self):

        localctx = SodaCLAntlrParser.Between_thresholdContext(self, self._ctx, self.state)
        self.enterRule(localctx, 40, self.RULE_between_threshold)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 212
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==25:
                self.state = 210
                self.match(SodaCLAntlrParser.NOT)
                self.state = 211
                self.match(SodaCLAntlrParser.S)


            self.state = 214
            self.match(SodaCLAntlrParser.BETWEEN)
            self.state = 215
            self.match(SodaCLAntlrParser.S)
            self.state = 217
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==35 or _la==39:
                self.state = 216
                _la = self._input.LA(1)
                if not(_la==35 or _la==39):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()


            self.state = 219
            self.threshold_value()
            self.state = 220
            self.match(SodaCLAntlrParser.S)
            self.state = 221
            self.match(SodaCLAntlrParser.AND)
            self.state = 222
            self.match(SodaCLAntlrParser.S)
            self.state = 223
            self.threshold_value()
            self.state = 225
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==36 or _la==40:
                self.state = 224
                _la = self._input.LA(1)
                if not(_la==36 or _la==40):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Comparator_thresholdContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def comparator(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.ComparatorContext,0)


        def S(self):
            return self.getToken(SodaCLAntlrParser.S, 0)

        def threshold_value(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Threshold_valueContext,0)


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_comparator_threshold

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterComparator_threshold" ):
                listener.enterComparator_threshold(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitComparator_threshold" ):
                listener.exitComparator_threshold(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitComparator_threshold" ):
                return visitor.visitComparator_threshold(self)
            else:
                return visitor.visitChildren(self)




    def comparator_threshold(self):

        localctx = SodaCLAntlrParser.Comparator_thresholdContext(self, self._ctx, self.state)
        self.enterRule(localctx, 42, self.RULE_comparator_threshold)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 227
            self.comparator()
            self.state = 228
            self.match(SodaCLAntlrParser.S)
            self.state = 229
            self.threshold_value()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Zones_thresholdContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def outcome(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SodaCLAntlrParser.OutcomeContext)
            else:
                return self.getTypedRuleContext(SodaCLAntlrParser.OutcomeContext,i)


        def S(self, i:int=None):
            if i is None:
                return self.getTokens(SodaCLAntlrParser.S)
            else:
                return self.getToken(SodaCLAntlrParser.S, i)

        def zone_comparator(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SodaCLAntlrParser.Zone_comparatorContext)
            else:
                return self.getTypedRuleContext(SodaCLAntlrParser.Zone_comparatorContext,i)


        def threshold_value(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SodaCLAntlrParser.Threshold_valueContext)
            else:
                return self.getTypedRuleContext(SodaCLAntlrParser.Threshold_valueContext,i)


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_zones_threshold

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterZones_threshold" ):
                listener.enterZones_threshold(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitZones_threshold" ):
                listener.exitZones_threshold(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitZones_threshold" ):
                return visitor.visitZones_threshold(self)
            else:
                return visitor.visitChildren(self)




    def zones_threshold(self):

        localctx = SodaCLAntlrParser.Zones_thresholdContext(self, self._ctx, self.state)
        self.enterRule(localctx, 44, self.RULE_zones_threshold)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 240 
            self._errHandler.sync(self)
            _alt = 1
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt == 1:
                    self.state = 231
                    self.outcome()
                    self.state = 232
                    self.match(SodaCLAntlrParser.S)
                    self.state = 233
                    self.zone_comparator()
                    self.state = 234
                    self.match(SodaCLAntlrParser.S)
                    self.state = 235
                    self.threshold_value()
                    self.state = 236
                    self.match(SodaCLAntlrParser.S)
                    self.state = 237
                    self.zone_comparator()
                    self.state = 238
                    self.match(SodaCLAntlrParser.S)

                else:
                    raise NoViableAltException(self)
                self.state = 242 
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,18,self._ctx)

            self.state = 244
            self.outcome()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class OutcomeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def WARN(self):
            return self.getToken(SodaCLAntlrParser.WARN, 0)

        def FAIL(self):
            return self.getToken(SodaCLAntlrParser.FAIL, 0)

        def PASS(self):
            return self.getToken(SodaCLAntlrParser.PASS, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_outcome

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterOutcome" ):
                listener.enterOutcome(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitOutcome" ):
                listener.exitOutcome(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitOutcome" ):
                return visitor.visitOutcome(self)
            else:
                return visitor.visitChildren(self)




    def outcome(self):

        localctx = SodaCLAntlrParser.OutcomeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 46, self.RULE_outcome)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 246
            _la = self._input.LA(1)
            if not(((_la) & ~0x3f) == 0 and ((1 << _la) & 939524096) != 0):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Zone_comparatorContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LT(self):
            return self.getToken(SodaCLAntlrParser.LT, 0)

        def LTE(self):
            return self.getToken(SodaCLAntlrParser.LTE, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_zone_comparator

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterZone_comparator" ):
                listener.enterZone_comparator(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitZone_comparator" ):
                listener.exitZone_comparator(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitZone_comparator" ):
                return visitor.visitZone_comparator(self)
            else:
                return visitor.visitChildren(self)




    def zone_comparator(self):

        localctx = SodaCLAntlrParser.Zone_comparatorContext(self, self._ctx, self.state)
        self.enterRule(localctx, 48, self.RULE_zone_comparator)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 248
            _la = self._input.LA(1)
            if not(_la==47 or _la==50):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ComparatorContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LT(self):
            return self.getToken(SodaCLAntlrParser.LT, 0)

        def LTE(self):
            return self.getToken(SodaCLAntlrParser.LTE, 0)

        def EQUAL(self):
            return self.getToken(SodaCLAntlrParser.EQUAL, 0)

        def GTE(self):
            return self.getToken(SodaCLAntlrParser.GTE, 0)

        def GT(self):
            return self.getToken(SodaCLAntlrParser.GT, 0)

        def NOT_EQUAL(self):
            return self.getToken(SodaCLAntlrParser.NOT_EQUAL, 0)

        def NOT_EQUAL_SQL(self):
            return self.getToken(SodaCLAntlrParser.NOT_EQUAL_SQL, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_comparator

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterComparator" ):
                listener.enterComparator(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitComparator" ):
                listener.exitComparator(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitComparator" ):
                return visitor.visitComparator(self)
            else:
                return visitor.visitChildren(self)




    def comparator(self):

        localctx = SodaCLAntlrParser.ComparatorContext(self, self._ctx, self.state)
        self.enterRule(localctx, 50, self.RULE_comparator)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 250
            _la = self._input.LA(1)
            if not(((_la) & ~0x3f) == 0 and ((1 << _la) & 4468415255281664) != 0):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Threshold_valueContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def signed_number(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Signed_numberContext,0)


        def PERCENT(self):
            return self.getToken(SodaCLAntlrParser.PERCENT, 0)

        def S(self):
            return self.getToken(SodaCLAntlrParser.S, 0)

        def freshness_threshold_value(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Freshness_threshold_valueContext,0)


        def IDENTIFIER_UNQUOTED(self):
            return self.getToken(SodaCLAntlrParser.IDENTIFIER_UNQUOTED, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_threshold_value

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterThreshold_value" ):
                listener.enterThreshold_value(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitThreshold_value" ):
                listener.exitThreshold_value(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitThreshold_value" ):
                return visitor.visitThreshold_value(self)
            else:
                return visitor.visitChildren(self)




    def threshold_value(self):

        localctx = SodaCLAntlrParser.Threshold_valueContext(self, self._ctx, self.state)
        self.enterRule(localctx, 52, self.RULE_threshold_value)
        self._la = 0 # Token type
        try:
            self.state = 261
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,21,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 252
                self.signed_number()
                self.state = 257
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,20,self._ctx)
                if la_ == 1:
                    self.state = 254
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==57:
                        self.state = 253
                        self.match(SodaCLAntlrParser.S)


                    self.state = 256
                    self.match(SodaCLAntlrParser.PERCENT)


                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 259
                self.freshness_threshold_value()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 260
                self.match(SodaCLAntlrParser.IDENTIFIER_UNQUOTED)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Freshness_threshold_valueContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def integer(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SodaCLAntlrParser.IntegerContext)
            else:
                return self.getTypedRuleContext(SodaCLAntlrParser.IntegerContext,i)


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_freshness_threshold_value

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFreshness_threshold_value" ):
                listener.enterFreshness_threshold_value(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFreshness_threshold_value" ):
                listener.exitFreshness_threshold_value(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFreshness_threshold_value" ):
                return visitor.visitFreshness_threshold_value(self)
            else:
                return visitor.visitChildren(self)




    def freshness_threshold_value(self):

        localctx = SodaCLAntlrParser.Freshness_threshold_valueContext(self, self._ctx, self.state)
        self.enterRule(localctx, 54, self.RULE_freshness_threshold_value)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 266 
            self._errHandler.sync(self)
            _alt = 1
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt == 1:
                    self.state = 263
                    self.integer()
                    self.state = 264
                    _la = self._input.LA(1)
                    if not(((_la) & ~0x3f) == 0 and ((1 << _la) & 7168) != 0):
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()

                else:
                    raise NoViableAltException(self)
                self.state = 268 
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,22,self._ctx)

            self.state = 271
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==56:
                self.state = 270
                self.integer()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Reference_checkContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def S(self, i:int=None):
            if i is None:
                return self.getTokens(SodaCLAntlrParser.S)
            else:
                return self.getToken(SodaCLAntlrParser.S, i)

        def source_column_name(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SodaCLAntlrParser.Source_column_nameContext)
            else:
                return self.getTypedRuleContext(SodaCLAntlrParser.Source_column_nameContext,i)


        def identifier(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.IdentifierContext,0)


        def target_column_name(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SodaCLAntlrParser.Target_column_nameContext)
            else:
                return self.getTypedRuleContext(SodaCLAntlrParser.Target_column_nameContext,i)


        def ROUND_LEFT(self, i:int=None):
            if i is None:
                return self.getTokens(SodaCLAntlrParser.ROUND_LEFT)
            else:
                return self.getToken(SodaCLAntlrParser.ROUND_LEFT, i)

        def ROUND_RIGHT(self, i:int=None):
            if i is None:
                return self.getTokens(SodaCLAntlrParser.ROUND_RIGHT)
            else:
                return self.getToken(SodaCLAntlrParser.ROUND_RIGHT, i)

        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(SodaCLAntlrParser.COMMA)
            else:
                return self.getToken(SodaCLAntlrParser.COMMA, i)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_reference_check

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterReference_check" ):
                listener.enterReference_check(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitReference_check" ):
                listener.exitReference_check(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitReference_check" ):
                return visitor.visitReference_check(self)
            else:
                return visitor.visitChildren(self)




    def reference_check(self):

        localctx = SodaCLAntlrParser.Reference_checkContext(self, self._ctx, self.state)
        self.enterRule(localctx, 56, self.RULE_reference_check)
        self._la = 0 # Token type
        try:
            self.state = 313
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,26,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 273
                self.match(SodaCLAntlrParser.T__12)
                self.state = 274
                self.match(SodaCLAntlrParser.S)
                self.state = 275
                self.source_column_name()
                self.state = 276
                self.match(SodaCLAntlrParser.S)
                self.state = 277
                self.match(SodaCLAntlrParser.T__13)
                self.state = 278
                self.match(SodaCLAntlrParser.S)
                self.state = 279
                self.identifier()
                self.state = 280
                self.match(SodaCLAntlrParser.S)
                self.state = 281
                self.target_column_name()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 283
                self.match(SodaCLAntlrParser.T__12)
                self.state = 284
                self.match(SodaCLAntlrParser.S)
                self.state = 285
                self.match(SodaCLAntlrParser.ROUND_LEFT)
                self.state = 286
                self.source_column_name()
                self.state = 292
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==41:
                    self.state = 287
                    self.match(SodaCLAntlrParser.COMMA)
                    self.state = 288
                    self.match(SodaCLAntlrParser.S)
                    self.state = 289
                    self.source_column_name()
                    self.state = 294
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 295
                self.match(SodaCLAntlrParser.ROUND_RIGHT)
                self.state = 296
                self.match(SodaCLAntlrParser.S)
                self.state = 297
                self.match(SodaCLAntlrParser.T__13)
                self.state = 298
                self.match(SodaCLAntlrParser.S)
                self.state = 299
                self.identifier()
                self.state = 300
                self.match(SodaCLAntlrParser.S)
                self.state = 301
                self.match(SodaCLAntlrParser.ROUND_LEFT)
                self.state = 302
                self.target_column_name()
                self.state = 308
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==41:
                    self.state = 303
                    self.match(SodaCLAntlrParser.COMMA)
                    self.state = 304
                    self.match(SodaCLAntlrParser.S)
                    self.state = 305
                    self.target_column_name()
                    self.state = 310
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 311
                self.match(SodaCLAntlrParser.ROUND_RIGHT)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Source_column_nameContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def identifier(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.IdentifierContext,0)


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_source_column_name

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSource_column_name" ):
                listener.enterSource_column_name(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSource_column_name" ):
                listener.exitSource_column_name(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSource_column_name" ):
                return visitor.visitSource_column_name(self)
            else:
                return visitor.visitChildren(self)




    def source_column_name(self):

        localctx = SodaCLAntlrParser.Source_column_nameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 58, self.RULE_source_column_name)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 315
            self.identifier()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Target_column_nameContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def identifier(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.IdentifierContext,0)


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_target_column_name

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTarget_column_name" ):
                listener.enterTarget_column_name(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTarget_column_name" ):
                listener.exitTarget_column_name(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTarget_column_name" ):
                return visitor.visitTarget_column_name(self)
            else:
                return visitor.visitChildren(self)




    def target_column_name(self):

        localctx = SodaCLAntlrParser.Target_column_nameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 60, self.RULE_target_column_name)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 317
            self.identifier()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Section_headerContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def table_checks_header(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Table_checks_headerContext,0)


        def column_configurations_header(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Column_configurations_headerContext,0)


        def table_filter_header(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Table_filter_headerContext,0)


        def checks_for_each_dataset_header(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Checks_for_each_dataset_headerContext,0)


        def checks_for_each_column_header(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Checks_for_each_column_headerContext,0)


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_section_header

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSection_header" ):
                listener.enterSection_header(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSection_header" ):
                listener.exitSection_header(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSection_header" ):
                return visitor.visitSection_header(self)
            else:
                return visitor.visitChildren(self)




    def section_header(self):

        localctx = SodaCLAntlrParser.Section_headerContext(self, self._ctx, self.state)
        self.enterRule(localctx, 62, self.RULE_section_header)
        try:
            self.state = 324
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [15]:
                self.enterOuterAlt(localctx, 1)
                self.state = 319
                self.table_checks_header()
                pass
            elif token in [17]:
                self.enterOuterAlt(localctx, 2)
                self.state = 320
                self.column_configurations_header()
                pass
            elif token in [16]:
                self.enterOuterAlt(localctx, 3)
                self.state = 321
                self.table_filter_header()
                pass
            elif token in [18, 19]:
                self.enterOuterAlt(localctx, 4)
                self.state = 322
                self.checks_for_each_dataset_header()
                pass
            elif token in [20]:
                self.enterOuterAlt(localctx, 5)
                self.state = 323
                self.checks_for_each_column_header()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Table_checks_headerContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def S(self, i:int=None):
            if i is None:
                return self.getTokens(SodaCLAntlrParser.S)
            else:
                return self.getToken(SodaCLAntlrParser.S, i)

        def identifier(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.IdentifierContext,0)


        def EOF(self):
            return self.getToken(SodaCLAntlrParser.EOF, 0)

        def partition_name(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Partition_nameContext,0)


        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_table_checks_header

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTable_checks_header" ):
                listener.enterTable_checks_header(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTable_checks_header" ):
                listener.exitTable_checks_header(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTable_checks_header" ):
                return visitor.visitTable_checks_header(self)
            else:
                return visitor.visitChildren(self)




    def table_checks_header(self):

        localctx = SodaCLAntlrParser.Table_checks_headerContext(self, self._ctx, self.state)
        self.enterRule(localctx, 64, self.RULE_table_checks_header)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 326
            self.match(SodaCLAntlrParser.T__14)
            self.state = 327
            self.match(SodaCLAntlrParser.S)
            self.state = 328
            self.identifier()
            self.state = 331
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==57:
                self.state = 329
                self.match(SodaCLAntlrParser.S)
                self.state = 330
                self.partition_name()


            self.state = 333
            self.match(SodaCLAntlrParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Partition_nameContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def SQUARE_LEFT(self):
            return self.getToken(SodaCLAntlrParser.SQUARE_LEFT, 0)

        def identifier(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.IdentifierContext,0)


        def SQUARE_RIGHT(self):
            return self.getToken(SodaCLAntlrParser.SQUARE_RIGHT, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_partition_name

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPartition_name" ):
                listener.enterPartition_name(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPartition_name" ):
                listener.exitPartition_name(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitPartition_name" ):
                return visitor.visitPartition_name(self)
            else:
                return visitor.visitChildren(self)




    def partition_name(self):

        localctx = SodaCLAntlrParser.Partition_nameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 66, self.RULE_partition_name)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 335
            self.match(SodaCLAntlrParser.SQUARE_LEFT)
            self.state = 336
            self.identifier()
            self.state = 337
            self.match(SodaCLAntlrParser.SQUARE_RIGHT)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Table_filter_headerContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def S(self, i:int=None):
            if i is None:
                return self.getTokens(SodaCLAntlrParser.S)
            else:
                return self.getToken(SodaCLAntlrParser.S, i)

        def identifier(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.IdentifierContext,0)


        def partition_name(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.Partition_nameContext,0)


        def EOF(self):
            return self.getToken(SodaCLAntlrParser.EOF, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_table_filter_header

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTable_filter_header" ):
                listener.enterTable_filter_header(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTable_filter_header" ):
                listener.exitTable_filter_header(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTable_filter_header" ):
                return visitor.visitTable_filter_header(self)
            else:
                return visitor.visitChildren(self)




    def table_filter_header(self):

        localctx = SodaCLAntlrParser.Table_filter_headerContext(self, self._ctx, self.state)
        self.enterRule(localctx, 68, self.RULE_table_filter_header)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 339
            self.match(SodaCLAntlrParser.T__15)
            self.state = 340
            self.match(SodaCLAntlrParser.S)
            self.state = 341
            self.identifier()
            self.state = 342
            self.match(SodaCLAntlrParser.S)
            self.state = 343
            self.partition_name()
            self.state = 344
            self.match(SodaCLAntlrParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Column_configurations_headerContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def S(self):
            return self.getToken(SodaCLAntlrParser.S, 0)

        def identifier(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.IdentifierContext,0)


        def EOF(self):
            return self.getToken(SodaCLAntlrParser.EOF, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_column_configurations_header

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterColumn_configurations_header" ):
                listener.enterColumn_configurations_header(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitColumn_configurations_header" ):
                listener.exitColumn_configurations_header(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitColumn_configurations_header" ):
                return visitor.visitColumn_configurations_header(self)
            else:
                return visitor.visitChildren(self)




    def column_configurations_header(self):

        localctx = SodaCLAntlrParser.Column_configurations_headerContext(self, self._ctx, self.state)
        self.enterRule(localctx, 70, self.RULE_column_configurations_header)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 346
            self.match(SodaCLAntlrParser.T__16)
            self.state = 347
            self.match(SodaCLAntlrParser.S)
            self.state = 348
            self.identifier()
            self.state = 349
            self.match(SodaCLAntlrParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Checks_for_each_dataset_headerContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def S(self):
            return self.getToken(SodaCLAntlrParser.S, 0)

        def identifier(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.IdentifierContext,0)


        def EOF(self):
            return self.getToken(SodaCLAntlrParser.EOF, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_checks_for_each_dataset_header

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterChecks_for_each_dataset_header" ):
                listener.enterChecks_for_each_dataset_header(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitChecks_for_each_dataset_header" ):
                listener.exitChecks_for_each_dataset_header(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitChecks_for_each_dataset_header" ):
                return visitor.visitChecks_for_each_dataset_header(self)
            else:
                return visitor.visitChildren(self)




    def checks_for_each_dataset_header(self):

        localctx = SodaCLAntlrParser.Checks_for_each_dataset_headerContext(self, self._ctx, self.state)
        self.enterRule(localctx, 72, self.RULE_checks_for_each_dataset_header)
        try:
            self.state = 361
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [18]:
                self.enterOuterAlt(localctx, 1)
                self.state = 351
                self.match(SodaCLAntlrParser.T__17)
                self.state = 352
                self.match(SodaCLAntlrParser.S)
                self.state = 353
                self.identifier()
                self.state = 354
                self.match(SodaCLAntlrParser.EOF)
                pass
            elif token in [19]:
                self.enterOuterAlt(localctx, 2)
                self.state = 356
                self.match(SodaCLAntlrParser.T__18)
                self.state = 357
                self.match(SodaCLAntlrParser.S)
                self.state = 358
                self.identifier()
                self.state = 359
                self.match(SodaCLAntlrParser.EOF)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Checks_for_each_column_headerContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def S(self):
            return self.getToken(SodaCLAntlrParser.S, 0)

        def identifier(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.IdentifierContext,0)


        def EOF(self):
            return self.getToken(SodaCLAntlrParser.EOF, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_checks_for_each_column_header

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterChecks_for_each_column_header" ):
                listener.enterChecks_for_each_column_header(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitChecks_for_each_column_header" ):
                listener.exitChecks_for_each_column_header(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitChecks_for_each_column_header" ):
                return visitor.visitChecks_for_each_column_header(self)
            else:
                return visitor.visitChildren(self)




    def checks_for_each_column_header(self):

        localctx = SodaCLAntlrParser.Checks_for_each_column_headerContext(self, self._ctx, self.state)
        self.enterRule(localctx, 74, self.RULE_checks_for_each_column_header)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 363
            self.match(SodaCLAntlrParser.T__19)
            self.state = 364
            self.match(SodaCLAntlrParser.S)
            self.state = 365
            self.identifier()
            self.state = 366
            self.match(SodaCLAntlrParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Signed_numberContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def number(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.NumberContext,0)


        def PLUS(self):
            return self.getToken(SodaCLAntlrParser.PLUS, 0)

        def MINUS(self):
            return self.getToken(SodaCLAntlrParser.MINUS, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_signed_number

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSigned_number" ):
                listener.enterSigned_number(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSigned_number" ):
                listener.exitSigned_number(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSigned_number" ):
                return visitor.visitSigned_number(self)
            else:
                return visitor.visitChildren(self)




    def signed_number(self):

        localctx = SodaCLAntlrParser.Signed_numberContext(self, self._ctx, self.state)
        self.enterRule(localctx, 76, self.RULE_signed_number)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 369
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==43 or _la==44:
                self.state = 368
                _la = self._input.LA(1)
                if not(_la==43 or _la==44):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()


            self.state = 371
            self.number()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class NumberContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def integer(self):
            return self.getTypedRuleContext(SodaCLAntlrParser.IntegerContext,0)


        def DIGITS(self, i:int=None):
            if i is None:
                return self.getTokens(SodaCLAntlrParser.DIGITS)
            else:
                return self.getToken(SodaCLAntlrParser.DIGITS, i)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_number

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNumber" ):
                listener.enterNumber(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNumber" ):
                listener.exitNumber(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitNumber" ):
                return visitor.visitNumber(self)
            else:
                return visitor.visitChildren(self)




    def number(self):

        localctx = SodaCLAntlrParser.NumberContext(self, self._ctx, self.state)
        self.enterRule(localctx, 78, self.RULE_number)
        self._la = 0 # Token type
        try:
            self.state = 384
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,33,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 373
                self.integer()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 374
                self.match(SodaCLAntlrParser.DIGITS)
                self.state = 375
                self.match(SodaCLAntlrParser.T__20)
                self.state = 377
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==56:
                    self.state = 376
                    self.match(SodaCLAntlrParser.DIGITS)


                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 380
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==56:
                    self.state = 379
                    self.match(SodaCLAntlrParser.DIGITS)


                self.state = 382
                self.match(SodaCLAntlrParser.T__20)
                self.state = 383
                self.match(SodaCLAntlrParser.DIGITS)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class IntegerContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def DIGITS(self):
            return self.getToken(SodaCLAntlrParser.DIGITS, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_integer

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterInteger" ):
                listener.enterInteger(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitInteger" ):
                listener.exitInteger(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitInteger" ):
                return visitor.visitInteger(self)
            else:
                return visitor.visitChildren(self)




    def integer(self):

        localctx = SodaCLAntlrParser.IntegerContext(self, self._ctx, self.state)
        self.enterRule(localctx, 80, self.RULE_integer)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 386
            self.match(SodaCLAntlrParser.DIGITS)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class IdentifierContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def IDENTIFIER_UNQUOTED(self):
            return self.getToken(SodaCLAntlrParser.IDENTIFIER_UNQUOTED, 0)

        def IDENTIFIER_DOUBLE_QUOTE(self):
            return self.getToken(SodaCLAntlrParser.IDENTIFIER_DOUBLE_QUOTE, 0)

        def IDENTIFIER_BACKTICK(self):
            return self.getToken(SodaCLAntlrParser.IDENTIFIER_BACKTICK, 0)

        def MIN(self):
            return self.getToken(SodaCLAntlrParser.MIN, 0)

        def MAX(self):
            return self.getToken(SodaCLAntlrParser.MAX, 0)

        def AVG(self):
            return self.getToken(SodaCLAntlrParser.AVG, 0)

        def getRuleIndex(self):
            return SodaCLAntlrParser.RULE_identifier

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterIdentifier" ):
                listener.enterIdentifier(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitIdentifier" ):
                listener.exitIdentifier(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitIdentifier" ):
                return visitor.visitIdentifier(self)
            else:
                return visitor.visitChildren(self)




    def identifier(self):

        localctx = SodaCLAntlrParser.IdentifierContext(self, self._ctx, self.state)
        self.enterRule(localctx, 82, self.RULE_identifier)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 388
            _la = self._input.LA(1)
            if not(((_la) & ~0x3f) == 0 and ((1 << _la) & 31525227456364544) != 0):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx








from soda.execution.query.query import Query


class TableColumnsQuery(Query):
    def __init__(self, partition: "Partition", schema_metric: "SchemaMetric"):
        super().__init__(
            data_source_scan=partition.data_source_scan,
            table=partition.table,
            partition=partition,
            unqualified_query_name=f"schema[{partition.table.table_name}]",
        )
        self.metric = schema_metric

    def execute(self):
        self._initialize_column_rows()
        self._propagate_column_rows_to_metric_value()

    def _initialize_column_rows(self):
        """
        Initializes member self.rows as a list (or tuple) of rows where each row representing a column description.
        A column description is a list (or tuple) of column name on index 0 and column data type (str) on index 1
        Eg [["col_name_one", "data_type_of_col_name_one"], ...]
        """
        data_source = self.data_source_scan.data_source
        self.sql = data_source.sql_get_table_columns(self.table.table_name)
        self.sql = self.data_source_scan.scan.jinja_resolve(self.sql)

        self.fetchall()

    def _propagate_column_rows_to_metric_value(self):
        """
        Propagates self.rows to the metric value being a dict with name and type as keys
        """
        if len(self.rows) > 0:
            measured_schema = [{"name": row[0], "type": row[1]} for row in self.rows]
            self.metric.set_value(measured_schema)



from __future__ import annotations


class DbtCloudConfig:
    def __init__(
        self,
        api_token: str | None,
        account_id: str | None,
        api_url: str | None = "https://cloud.getdbt.com/api/v2/accounts/",
    ):
        self.api_token = api_token
        self.account_id = account_id
        self.api_url = api_url



from soda.execution.metric.query_metric import QueryMetric
from soda.execution.query.user_defined_failed_rows_query import (
    UserDefinedFailedRowsQuery,
)


class UserDefinedFailedRowsMetric(QueryMetric):
    def __init__(
        self,
        data_source_scan: "DataSourceScan",
        check_name: str,
        query: str,
        check: "Check",
        partition: "Partition",
    ):
        super().__init__(
            data_source_scan=data_source_scan,
            partition=partition,
            column=None,
            name=check_name,
            check=check,
            identity_parts=[query],
        )

        self.query: str = query

    def __str__(self):
        return f'"{self.name}"'

    def set_value(self, value):
        if value is None or isinstance(value, int):
            self.value = value
        else:
            self.value = int(value)

    def ensure_query(self):
        check = next(iter(self.checks), None)
        location = check.check_cfg.location if check else None
        self.data_source_scan.queries.append(
            UserDefinedFailedRowsQuery(
                data_source_scan=self.data_source_scan,
                metric=self,
                location=location,
                samples_limit=check.check_cfg.samples_limit,
                partition=self.partition,
            )
        )



from dataclasses import dataclass
from typing import Dict, List, Optional

from soda.sodacl.check_cfg import CheckCfg
from soda.sodacl.location import Location


@dataclass
class SchemaValidations:
    required_column_names: Optional[List[str]]
    required_column_types: Optional[Dict[str, str]]
    required_column_indexes: Optional[Dict[str, int]]
    forbidden_column_names: Optional[List[str]]
    is_column_addition_forbidden: bool
    is_column_deletion_forbidden: bool
    is_column_type_change_forbidden: bool
    is_column_index_change_forbidden: bool

    def has_change_validations(self):
        return (
            self.is_column_addition_forbidden
            or self.is_column_deletion_forbidden
            or self.is_column_type_change_forbidden
            or self.is_column_index_change_forbidden
        )


class SchemaCheckCfg(CheckCfg):
    def __init__(
        self,
        source_header: str,
        source_line: str,
        source_configurations: Optional[str],
        location: Location,
        name: Optional[str],
        warn_validations: SchemaValidations,
        fail_validations: SchemaValidations,
        is_automated_monitoring: bool = False,
    ):
        super().__init__(source_header, source_line, source_configurations, location, name)
        self.warn_validations: SchemaValidations = warn_validations
        self.fail_validations: SchemaValidations = fail_validations
        self.is_automated_monitoring = is_automated_monitoring
        self.name = name if name else "Schema Check"

    def has_change_validations(self) -> bool:
        return (self.warn_validations and self.warn_validations.has_change_validations()) or (
            self.fail_validations and self.fail_validations.has_change_validations()
        )



from __future__ import annotations

from dataclasses import dataclass
from datetime import timedelta

from soda.execution.identity import Identity


@dataclass
class ThresholdCfg:
    # value must be greater than or equal to be ok
    gte: float | str = None
    # value must be greater than to be ok
    gt: float | str = None
    # value must be less than to be ok
    lt: float | str = None
    # value must be less than or equal to be ok
    lte: float | str = None

    is_split_zone: bool = False

    def get_identity_parts(self) -> list:
        return [
            Identity.property("gte", self.gte),
            Identity.property("gt", self.gt),
            Identity.property("lt", self.lt),
            Identity.property("lte", self.lte),
            Identity.property("is_split_zone", self.is_split_zone),
        ]

    def get_freshness_threshold(self) -> timedelta | None:
        if isinstance(self.gte, timedelta):
            return self.gte
        elif isinstance(self.gt, timedelta):
            return self.gt

    def get_inverse(self) -> ThresholdCfg:
        has_lower_limit = self.gt is not None or self.gte is not None
        has_upper_limit = self.lt is not None or self.lte is not None
        new_is_split_zone = not self.is_split_zone if has_lower_limit and has_upper_limit else self.is_split_zone
        return ThresholdCfg(
            gte=self.lt if self.lt is not None else None,
            gt=self.lte if self.lte is not None else None,
            lt=self.gte if self.gte is not None else None,
            lte=self.gt if self.gt is not None else None,
            is_split_zone=new_is_split_zone,
        )

    def resolve(self, f):
        """
        Resolve if the values are set to be strings
        :param f:
        :return:
        """
        if isinstance(self.lt, str):
            self.lt = float(f(self.lt))
        if isinstance(self.gte, str):
            self.gte = float(f(self.gte))
        if isinstance(self.gt, str):
            self.gt = float(f(self.gt))
        if isinstance(self.lte, str):
            self.lte = float(f(self.lte))
        return self

    def is_bad(self, value) -> bool:
        if self.is_split_zone:
            return ((self.gte is None or value >= self.gte) and (self.gt is None or value > self.gt)) or (
                (self.lt is None or value < self.lt) and (self.lte is None or value <= self.lte)
            )
        else:
            return (
                (self.gte is None or value >= self.gte)
                and (self.gt is None or value > self.gt)
                and (self.lt is None or value < self.lt)
                and (self.lte is None or value <= self.lte)
            )

    def is_valid_anomaly_threshold(self) -> bool:
        lower_limit = self.gte if self.gte is not None else self.gt
        return (
            isinstance(lower_limit, float)
            and 0 <= lower_limit <= 1
            and self.lt is None
            and self.lte is None
            and self.is_split_zone is False
        )

    def to_soda_cloud_diagnostics_json(self):
        json = {}
        if self.gte is not None:
            json["greaterThanOrEqual"] = self.gte
        if self.gt is not None:
            json["greaterThan"] = self.gt
        if self.lt is not None:
            json["lessThan"] = self.lt
        if self.lte is not None:
            json["lessThanOrEqual"] = self.lte
        return json



# Generated from /Users/vijay/work/soda/code/soda-core/soda/core/soda/sodacl/antlr/SodaCLAntlr.g4 by ANTLR 4.11.1
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .SodaCLAntlrParser import SodaCLAntlrParser
else:
    from SodaCLAntlrParser import SodaCLAntlrParser

# This class defines a complete generic visitor for a parse tree produced by SodaCLAntlrParser.

class SodaCLAntlrVisitor(ParseTreeVisitor):

    # Visit a parse tree produced by SodaCLAntlrParser#check.
    def visitCheck(self, ctx:SodaCLAntlrParser.CheckContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#freshness_check.
    def visitFreshness_check(self, ctx:SodaCLAntlrParser.Freshness_checkContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#freshness_variable.
    def visitFreshness_variable(self, ctx:SodaCLAntlrParser.Freshness_variableContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#warn_qualifier.
    def visitWarn_qualifier(self, ctx:SodaCLAntlrParser.Warn_qualifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#failed_rows_check.
    def visitFailed_rows_check(self, ctx:SodaCLAntlrParser.Failed_rows_checkContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#group_by_check.
    def visitGroup_by_check(self, ctx:SodaCLAntlrParser.Group_by_checkContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#row_count_comparison_check.
    def visitRow_count_comparison_check(self, ctx:SodaCLAntlrParser.Row_count_comparison_checkContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#metric_check.
    def visitMetric_check(self, ctx:SodaCLAntlrParser.Metric_checkContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#default_anomaly_threshold.
    def visitDefault_anomaly_threshold(self, ctx:SodaCLAntlrParser.Default_anomaly_thresholdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#change_over_time.
    def visitChange_over_time(self, ctx:SodaCLAntlrParser.Change_over_timeContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#change_over_time_config.
    def visitChange_over_time_config(self, ctx:SodaCLAntlrParser.Change_over_time_configContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#change_aggregation.
    def visitChange_aggregation(self, ctx:SodaCLAntlrParser.Change_aggregationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#same_day_last_week.
    def visitSame_day_last_week(self, ctx:SodaCLAntlrParser.Same_day_last_weekContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#percent.
    def visitPercent(self, ctx:SodaCLAntlrParser.PercentContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#anomaly_score.
    def visitAnomaly_score(self, ctx:SodaCLAntlrParser.Anomaly_scoreContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#metric.
    def visitMetric(self, ctx:SodaCLAntlrParser.MetricContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#metric_name.
    def visitMetric_name(self, ctx:SodaCLAntlrParser.Metric_nameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#metric_args.
    def visitMetric_args(self, ctx:SodaCLAntlrParser.Metric_argsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#metric_arg.
    def visitMetric_arg(self, ctx:SodaCLAntlrParser.Metric_argContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#threshold.
    def visitThreshold(self, ctx:SodaCLAntlrParser.ThresholdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#between_threshold.
    def visitBetween_threshold(self, ctx:SodaCLAntlrParser.Between_thresholdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#comparator_threshold.
    def visitComparator_threshold(self, ctx:SodaCLAntlrParser.Comparator_thresholdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#zones_threshold.
    def visitZones_threshold(self, ctx:SodaCLAntlrParser.Zones_thresholdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#outcome.
    def visitOutcome(self, ctx:SodaCLAntlrParser.OutcomeContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#zone_comparator.
    def visitZone_comparator(self, ctx:SodaCLAntlrParser.Zone_comparatorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#comparator.
    def visitComparator(self, ctx:SodaCLAntlrParser.ComparatorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#threshold_value.
    def visitThreshold_value(self, ctx:SodaCLAntlrParser.Threshold_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#freshness_threshold_value.
    def visitFreshness_threshold_value(self, ctx:SodaCLAntlrParser.Freshness_threshold_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#reference_check.
    def visitReference_check(self, ctx:SodaCLAntlrParser.Reference_checkContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#source_column_name.
    def visitSource_column_name(self, ctx:SodaCLAntlrParser.Source_column_nameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#target_column_name.
    def visitTarget_column_name(self, ctx:SodaCLAntlrParser.Target_column_nameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#section_header.
    def visitSection_header(self, ctx:SodaCLAntlrParser.Section_headerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#table_checks_header.
    def visitTable_checks_header(self, ctx:SodaCLAntlrParser.Table_checks_headerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#partition_name.
    def visitPartition_name(self, ctx:SodaCLAntlrParser.Partition_nameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#table_filter_header.
    def visitTable_filter_header(self, ctx:SodaCLAntlrParser.Table_filter_headerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#column_configurations_header.
    def visitColumn_configurations_header(self, ctx:SodaCLAntlrParser.Column_configurations_headerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#checks_for_each_dataset_header.
    def visitChecks_for_each_dataset_header(self, ctx:SodaCLAntlrParser.Checks_for_each_dataset_headerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#checks_for_each_column_header.
    def visitChecks_for_each_column_header(self, ctx:SodaCLAntlrParser.Checks_for_each_column_headerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#signed_number.
    def visitSigned_number(self, ctx:SodaCLAntlrParser.Signed_numberContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#number.
    def visitNumber(self, ctx:SodaCLAntlrParser.NumberContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#integer.
    def visitInteger(self, ctx:SodaCLAntlrParser.IntegerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by SodaCLAntlrParser#identifier.
    def visitIdentifier(self, ctx:SodaCLAntlrParser.IdentifierContext):
        return self.visitChildren(ctx)



del SodaCLAntlrParser